// Copyright 2018-present Network Optix, Inc. Licensed under MPL 2.0: www.mozilla.org/MPL/2.0/

#pragma once

#include <thread>
#include <atomic>
#include <memory>
#include <condition_variable>
#include <vector>
#include <string>
#include <deque>

#include <nx/sdk/analytics/helpers/consuming_device_agent.h>
#include <nx/sdk/analytics/helpers/pixel_format.h>
#include <nx/sdk/analytics/helpers/object_metadata_packet.h>

#include "engine.h"
#include "stub_analytics_plugin_diagnostic_events_ini.h"

namespace nx {
namespace vms_server_plugins {
namespace analytics {
namespace stub {
namespace diagnostic_events {

const std::string kThrowPluginDiagnosticEventsFromDeviceAgentSetting =
    "throwPluginDiagnosticEventsFromDeviceAgent";

class DeviceAgent: public nx::sdk::analytics::ConsumingDeviceAgent
{
public:
    DeviceAgent(Engine* engine, const nx::sdk::IDeviceInfo* deviceInfo);
    virtual ~DeviceAgent() override;

protected:
    virtual void doSetNeededMetadataTypes(
        nx::sdk::Result<void>* outValue,
        const nx::sdk::analytics::IMetadataTypes* neededMetadataTypes) override;

    virtual std::string manifestString() const override;

    virtual nx::sdk::Result<const nx::sdk::ISettingsResponse*> settingsReceived() override;

private:
    void pluginDiagnosticEventsThreadLoop();

    void processFrameMotion(
        nx::sdk::Ptr<nx::sdk::IList<nx::sdk::analytics::IMetadataPacket>> metadataPacketList);

private:
    Engine* const m_engine;

    std::atomic<bool> m_terminated{false};

    std::unique_ptr<std::thread> m_pluginDiagnosticEventThread;
    std::mutex m_pluginDiagnosticEventGenerationLoopMutex;
    std::condition_variable m_pluginDiagnosticEventGenerationLoopCondition;
    std::atomic<bool> m_needToThrowPluginDiagnosticEvents{false};

    struct DeviceAgentSettings
    {
        std::atomic<bool> throwPluginDiagnosticEvents{false};
    };

    DeviceAgentSettings m_deviceAgentSettings;
};

} // namespace diagnostic_events
} // namespace stub
} // namespace analytics
} // namespace vms_server_plugins
} // namespace nx
